/*
 * Copyright (C) 2018 - Timo Könnecke <el-t-mo@arcor.de>
 *
 * All rights reserved.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 2.1 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

import QtQuick 2.2


Rectangle {
    id: gaugeContainer
    width: 600 //parent.width
    height: 600 //parent.height

    property int maxSpeed: 220
    property int curSpeed: 218
    property int lastSpeed: 160

    Canvas {
        z: 1
        anchors.fill: parent
        smooth: true
        onPaint: {
            var ctx = getContext("2d")
            ctx.beginPath()
            ctx.fillStyle = Qt.rgba(0.0, 0.0, 0.0, 0.2)

            ctx.arc(gaugeContainer.width/2, gaugeContainer.height/2, gaugeContainer.height*0.42, 0, 2*Math.PI, false)
            ctx.closePath()
            ctx.fill()
            ctx.fillStyle = Qt.rgba(1.0, 1.0, 1.0, 0.7)
            ctx.beginPath()
            ctx.arc(gaugeContainer.width/2, gaugeContainer.height/2, gaugeContainer.height*0.375, 0, 2*Math.PI, false)
            ctx.closePath()
            ctx.fill()

        }
    }

    // strokes
    Canvas {
        z: 2
        anchors.fill: parent
        smooth: true
        onPaint: {
            var ctx = getContext("2d")
            ctx.lineWidth = gaugeContainer.width*0.015
            ctx.strokeStyle = Qt.rgba(0, 0, 0, 0.85)
            ctx.translate(gaugeContainer.width/2, gaugeContainer.height/2)
            ctx.rotate(Math.PI/3.9)

            for (var i=0; i < 23; i++) {
                // do not paint a minute stroke when there is an hour stroke
                    ctx.beginPath()
                    ctx.moveTo(0, gaugeContainer.height*0.42)
                    ctx.lineTo(0, gaugeContainer.height*0.375)
                    ctx.stroke()
                ctx.rotate(Math.PI/14.8)
            }
        }
    }

    ShaderEffect {
        z: 3
        id: gauge
        anchors.fill: parent
        opacity: 0.75

        property real angleBase: -pi*0.75
        property real angle

        readonly property real pi: 3.141592

        vertexShader: "
      uniform highp mat4 qt_Matrix;
      attribute highp vec4 qt_Vertex;
      attribute highp vec2 qt_MultiTexCoord0;
      varying highp vec2 coord;
      void main() {
        coord = qt_MultiTexCoord0;
        gl_Position = qt_Matrix * qt_Vertex;
      }"

        fragmentShader: "
      uniform lowp float qt_Opacity;
      uniform highp float angleBase;
      uniform highp float angle;
      varying highp vec2 coord;
      void main() {
        gl_FragColor = vec4(0.0,0.0,0.0,0.0);
        highp vec2 d=2.0*coord-vec2(1.0,1.0);
        highp float r=length(d);
        if (0.66<=r && r<=0.92) {
          highp float a=atan2(d.x,-d.y);
          if (angleBase<=a && a<=angle) {
            highp float p=(a-angleBase)/(angle-angleBase);
            gl_FragColor = vec4(0.9,0.2,0.1,p) * qt_Opacity;
          }
        }
      }"
    }

    SequentialAnimation {
        id: anim
        running: true
        NumberAnimation {
            from: gauge.angleBase+(lastSpeed*(1.5/maxSpeed))*gauge.pi
            to: gauge.angleBase+(curSpeed*(1.5/maxSpeed))*gauge.pi
            duration: 3000
            target: gauge
            property: 'angle'
            easing.type: Easing.InOutSine
        }
    }

    Text {
        z: 4
        id: curSpeedDisplay

        x: gaugeContainer.width/2-curSpeedDisplay.width/2
        y: gaugeContainer.height/2-curSpeedDisplay.height/1.8

        font.pixelSize: gaugeContainer.height*0.37
        font.family: "Source Han Sans"
        font.styleName: "Bold"
        font.letterSpacing: -gaugeContainer.height*0.015

        opacity: 0.90
        color: "#000000"
        text: curSpeed
    }
}
